@@ -128,6 +128,12 @@ def find_in_path(name):
             return val
     return name
 
+def test_server_version(host, port):
+    version = rpc.server_version(host, port)
+    if not version:
+        return False
+    return version.split('.')[:2] == VERSION.split('.')[:2]
+
 def refresh_dblist(host, port):
     '''
     Return the number of database available
@@ -135,10 +141,8 @@ def refresh_dblist(host, port):
         or -1 if the server version doesn't match the client version
     '''
     rpc.logout()
-    version = rpc.server_version(host, port)
-    if hasattr(version, 'split'):
-        if version.split('.')[:2] != VERSION.split('.')[:2]:
-            return -1
+    if not test_server_version(host, port):
+        return -1
     return rpc.db_list(host, port)
 
 def refresh_langlist(lang_widget, host, port):
@@ -520,9 +524,9 @@ message = MessageDialog()
 
 class WarningDialog(UniqueDialog):
 
-    def build_dialog(self, message, parent, title):
+    def build_dialog(self, message, parent, title, buttons=gtk.BUTTONS_OK):
         dialog = gtk.MessageDialog(parent, gtk.DIALOG_DESTROY_WITH_PARENT,
-            gtk.MESSAGE_WARNING, gtk.BUTTONS_OK)
+            gtk.MESSAGE_WARNING, buttons)
         if hasattr(dialog, 'format_secondary_markup'):
             dialog.set_markup('<b>%s</b>' % (to_xml(title)))
             dialog.format_secondary_markup(to_xml(message))
@@ -544,7 +548,7 @@ class UserWarningDialog(WarningDialog):
 
     def build_dialog(self, message, parent, title):
         dialog = super(UserWarningDialog, self).build_dialog(message, parent,
-            title)
+            title, gtk.BUTTONS_OK_CANCEL)
         check = gtk.CheckButton(_('Always ignore this warning.'))
         check.connect_after('toggled', self._set_always)
         alignment = gtk.Alignment(1, 0.5)
@@ -1080,16 +1084,13 @@ class DBProgress(object):
 
     def start(self):
         key = (self.host, self.port)
+        dbs, createdb = [], False
         try:
             dbs = refresh_dblist(self.host, self.port)
             createdb = True
-        except Exception, exception:
-            if exception[0] == 'AccessDenied':
-                dbs, createdb = [], False
-            else:
-                raise
-        self.db_info = (dbs, createdb)
-        self.updated.set()
+        finally:
+            self.db_info = (dbs, createdb)
+            self.updated.set()
 
     def update(self, combo, progressbar, dbname=''):
         key = (self.host, self.port)
@@ -1333,17 +1334,13 @@ def float_time_to_text(val, conv=None):
     val = val - hours
     mins = int((val% 1 + 0.01) / conv['m'])
     if years:
-        value += ' ' + locale.format('%d' + FLOAT_TIME_SEPS['Y'],
-                years, True)
+        value += ' ' + locale.format('%d', years, True) + FLOAT_TIME_SEPS['Y']
     if months:
-        value += ' ' + locale.format('%d' + FLOAT_TIME_SEPS['M'],
-                months, True)
+        value += ' ' + locale.format('%d', months, True) + FLOAT_TIME_SEPS['M']
     if weeks:
-        value += ' ' + locale.format('%d' + FLOAT_TIME_SEPS['w'],
-                weeks, True)
+        value += ' ' + locale.format('%d', weeks, True) + FLOAT_TIME_SEPS['w']
     if days:
-        value += ' ' + locale.format('%d' + FLOAT_TIME_SEPS['d'],
-                days, True)
+        value += ' ' + locale.format('%d', days, True) + FLOAT_TIME_SEPS['d']
     if hours or mins:
         value += ' %02d:%02d' % (hours, mins)
     value = value.strip()
