@@ -85,6 +85,7 @@ class DBListEditor(object):
         self.port_entry = gtk.Entry()
         self.port_entry.connect('focus-out-event', self.display_dbwidget)
         self.port_entry.connect('changed', self.update_profiles, 'port')
+        self.port_entry.connect('insert_text', self.insert_text_port)
         self.port_entry.set_activates_default(True)
         table.attach(port, 0, 1, 2, 3, yoptions=False, xoptions=gtk.FILL)
         table.attach(self.port_entry, 1, 2, 2, 3, yoptions=False)
@@ -142,8 +143,8 @@ class DBListEditor(object):
         self.dialog.set_default_response(gtk.RESPONSE_ACCEPT)
 
     def run(self, profile_name):
+        self.clear_entries()  # must be done before show_all for windows
         self.dialog.show_all()
-        self.clear_entries()
         model = self.profile_tree.get_model()
         if model:
             for i, row in enumerate(model):
@@ -344,6 +345,15 @@ class DBListEditor(object):
             self.profiles.set(self.current_profile['name'], 'database', dbname)
             self.validate_profile(self.current_profile['name'])
 
+    def insert_text_port(self, entry, new_text, new_text_length, position):
+        value = entry.get_text()
+        position = entry.get_position()
+        new_value = value[:position] + new_text + value[position:]
+        try:
+            int(new_value)
+        except ValueError:
+            entry.stop_emission('insert-text')
+
 
 class DBLogin(object):
     def __init__(self, parent):
@@ -478,7 +488,7 @@ class DBLogin(object):
             self.profiles.write(configfile)
 
         for idx, row in enumerate(self.profile_store):
-            if row[0] == profile_name:
+            if row[0] == profile_name and row[1]:
                 self.combo_profile.set_active(idx)
                 self.profile_changed(self.combo_profile)
                 break
@@ -493,6 +503,8 @@ class DBLogin(object):
         except ConfigParser.NoOptionError:
             username = ''
         host = self.profiles.get(profile, 'host')
+        if ':' in host:
+            host = '[%s]' % host
         port = self.profiles.get(profile, 'port')
         self.entry_host.set_text('%s:%s' % (host, port))
         self.entry_database.set_text(self.profiles.get(profile, 'database'))
@@ -510,10 +522,17 @@ class DBLogin(object):
         self.move_active = True
 
     def clear_profile_combo(self, entry, event):
-        host_entry = self.entry_host.get_text()
-        host, port = host_entry.split(':', 1) if ':' in host_entry else ('', '')
+        netloc = self.entry_host.get_text()
+        host = self.get_hostname(netloc)
+        try:
+            port = str(self.get_port(netloc))
+        except ValueError:
+            host = ''
+            port = ''
         database = self.entry_database.get_text().strip()
         for idx, profile_info in enumerate(self.profile_store):
+            if not profile_info[1]:
+                continue
             profile = profile_info[0]
             if (host == self.profiles.get(profile, 'host')
                     and port == self.profiles.get(profile, 'port')
@@ -530,6 +549,21 @@ class DBLogin(object):
         self.entry_database.props.visible = visibility
         self.label_database.props.visible = visibility
 
+    def get_hostname(self, netloc):
+        if '[' in netloc and ']' in netloc:
+            return netloc.split(']')[0][1:]
+        elif ':' in netloc:
+            return netloc.split(':')[0]
+        else:
+            return netloc
+
+    def get_port(self, netloc):
+        netloc = netloc.split(']')[-1]
+        if ':' in netloc:
+            return int(netloc.split(':')[1])
+        else:
+            return 8070
+
     def run(self, parent):
         profile_name = CONFIG['login.profile']
         can_use_profile = self.profiles.has_section(profile_name)
@@ -548,7 +582,11 @@ class DBLogin(object):
                     break
         else:
             self.combo_profile.set_active(-1)
-            self.entry_host.set_text('%s:%s' % (CONFIG['login.server'],
+            if ':' in CONFIG['login.server']:
+                host = '[%s]' % CONFIG['login.server']
+            else:
+                host = CONFIG['login.server']
+            self.entry_host.set_text('%s:%s' % (host,
                 CONFIG['login.port']))
             db = CONFIG['login.db'] if CONFIG['login.db'] else ''
             self.entry_database.set_text(db)
@@ -567,15 +605,26 @@ class DBLogin(object):
         self.expand_hostspec(self.expander)
 
         res, result = None, ('', '', '', '', '')
-        while not (res in (gtk.RESPONSE_CANCEL, gtk.RESPONSE_DELETE_EVENT)
-            or (res == gtk.RESPONSE_OK and all(result))):
+        while not all(result):
             res = self.dialog.run()
+            if res != gtk.RESPONSE_OK:
+                break
             active_profile = self.combo_profile.get_active()
             if active_profile != -1:
                 profile = self.profile_store[active_profile][0]
                 CONFIG['login.profile'] = profile
-            host, port = (self.entry_host.get_text().split(':', 1)
-                + ['8070'])[:2]
+            netloc = self.entry_host.get_text()
+            host = self.get_hostname(netloc)
+            try:
+                port = self.get_port(netloc)
+            except ValueError:
+                continue
+            if ':' in host:
+                hostname = '[%s]' % host
+            else:
+                hostname = host
+            if not common.test_server_version(host, port):
+                continue
             database = self.entry_database.get_text()
             login = self.entry_login.get_text()
             CONFIG['login.server'] = host
@@ -584,7 +633,7 @@ class DBLogin(object):
             CONFIG['login.expanded'] = self.expander.props.expanded
             CONFIG['login.login'] = login
             result = (self.entry_login.get_text(),
-                self.entry_password.get_text(), host, int(port), database)
+                self.entry_password.get_text(), host, port, database)
 
         if res != gtk.RESPONSE_OK:
             parent.present()
