@@ -8,17 +8,18 @@ import time
 import datetime
 from tryton.gui.window.win_search import WinSearch
 from tryton.action import Action as Action2
-from tryton.gui.window.view_tree.view_tree import ViewTree
 import tryton.common as common
 from tryton.pyson import PYSONDecoder
 import gettext
 from tryton.config import CONFIG
+from tryton.signal_event import SignalEvent
 _ = gettext.gettext
 
 
-class Action(object):
+class Action(SignalEvent):
 
     def __init__(self, window, attrs=None):
+        super(Action, self).__init__()
         self.act_id = int(attrs['name'])
         self._window = window
         self.screen = None
@@ -43,7 +44,8 @@ class Action(object):
             self.action['view_mode'] = attrs['view_mode']
 
         self.action.setdefault('pyson_domain', '[]')
-        self.context = {'active_id': False, 'active_ids': []}
+        self.context = {}
+        self.context.update(rpc.CONTEXT)
         self.context.update(PYSONDecoder(self.context).decode(
             self.action.get('pyson_context', '{}')))
 
@@ -51,10 +53,8 @@ class Action(object):
         self.context.update(PYSONDecoder(eval_ctx).decode(
             self.action.get('pyson_context', '{}')))
 
-        domain_ctx = self.context.copy()
-        self.domain = PYSONDecoder(domain_ctx).decode(
-                self.action['pyson_domain'])
-
+        self.domain = []
+        self.update_domain([])
 
         self.widget = gtk.Frame()
         self.widget.set_border_width(0)
@@ -73,7 +73,7 @@ class Action(object):
 
         tooltips = common.Tooltips()
 
-        if self.action['view_type'] == 'form':
+        if self.action['res_model']:
             but_search = gtk.Button()
             tooltips.set_tip(but_search, _('Search'))
             but_search.connect('clicked', self._sig_search)
@@ -96,7 +96,7 @@ class Action(object):
         hbox.pack_start(but_open, expand=False, fill=False)
 
 
-        if self.action['view_type'] == 'form':
+        if self.action['res_model']:
             hbox.pack_start(gtk.VSeparator(), expand=False, fill=True)
             but_previous = gtk.Button()
             tooltips.set_tip(but_previous, _('Previous'))
@@ -140,38 +140,20 @@ class Action(object):
         alignment.set_padding(0, 2, 2, 2)
         vbox.pack_start(alignment, expand=True, fill=True)
 
+        hbox.set_focus_chain([])
+
         self.widget.show_all()
 
-        if self.action['view_type'] == 'form':
-            self.screen = Screen(self.action['res_model'], self._window,
-                    view_type=self.action['view_mode'], context=self.context,
-                    view_ids=view_ids, domain=self.domain, readonly=True)
-            self.screen.screen_container.alternate_view = True
-            self.screen.switch_view(view_type=self.action['view_mode'])
+        if self.action['res_model']:
+            self.screen = Screen(self.action['res_model'],
+                self._window, mode=self.action['view_mode'],
+                context=self.context, view_ids=view_ids,
+                domain=self.domain, readonly=True, alternate_view=True)
             alignment.add(self.screen.screen_container.alternate_viewport)
             name = self.screen.current_view.title
             self.screen.signal_connect(self, 'record-message', self._sig_label)
-        elif self.action['view_type'] == 'tree':
-            ctx = {}
-            ctx.update(rpc.CONTEXT)
-            ctx.update(self.context)
-            try:
-                view_base = rpc.execute('model', 'ir.ui.view', 'read',
-                        view_ids[0], ['model', 'type'], ctx)
-            except Exception, exception:
-                common.process_exception(exception, self._window)
-                raise
-            try:
-                view = rpc.execute('model', view_base['model'],
-                        'fields_view_get', view_ids[0], view_base['type'], ctx)
-            except Exception, exception:
-                common.process_exception(exception, self._window)
-                raise
-            self.tree = ViewTree(view, [], self._window, True,
-                    context=ctx)
-            alignment.add(self.tree.widget_get())
-            name = self.tree.name
-            self.tree.view.connect('key_press_event', self.sig_key_press)
+            self.screen.signal_connect(self, 'record-message',
+                    self._active_changed)
 
         if attrs.get('string'):
             self.title.set_text(attrs['string'])
@@ -216,7 +198,7 @@ class Action(object):
     def _sig_label(self, screen, signal_data):
         name = '_'
         if signal_data[0] >= 0:
-            name = str(signal_data[0] + 1)
+            name = str(signal_data[0])
         line = '(%s/%s)' % (name, signal_data[1])
         self.label.set_text(line)
 
@@ -251,3 +233,29 @@ class Action(object):
                     .get_selected_rows()
             for path in paths:
                 self.tree.view.expand_row(path, False)
+
+    def _active_changed(self, *args):
+        self.signal('active-changed')
+
+    def _get_active(self):
+        if self.screen and self.screen.current_record:
+            return common.EvalEnvironment(self.screen.current_record, False)
+        elif self.tree:
+            return {'id': self.tree.sel_id_get()}
+
+    active = property(_get_active)
+
+    def update_domain(self, actions):
+        domain_ctx = self.context.copy()
+        domain_ctx['context'] = domain_ctx
+        for action in actions:
+            if action.active:
+                domain_ctx['_active_%s' % action.act_id] = action.active
+        new_domain = PYSONDecoder(domain_ctx).decode(
+                self.action['pyson_domain'])
+        if self.domain == new_domain:
+            return
+        del self.domain[:]
+        self.domain.extend(new_domain)
+        if self.screen:
+            self.screen.search_filter()
