@@ -1,7 +1,9 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
 import tryton.rpc as rpc
-from tryton.common import DT_FORMAT, DHM_FORMAT, HM_FORMAT, datetime_strftime
+from tryton.common import DT_FORMAT, DHM_FORMAT, HM_FORMAT, datetime_strftime, \
+        domain_inversion, eval_domain, localize_domain, unlocalize_domain, \
+        merge, inverse_leaf, EvalEnvironment
 import tryton.common as common
 from tryton.pyson import PYSONDecoder
 import time
@@ -25,8 +27,7 @@ class Field(object):
 
 class CharField(object):
 
-    def __init__(self, parent, attrs):
-        self.parent = parent
+    def __init__(self, attrs):
         self.attrs = attrs
         self.name = attrs['name']
         self.internal = False
@@ -37,36 +38,87 @@ class CharField(object):
             return
         if self.attrs.get('on_change', False):
             record.on_change(self.name, self.attrs['on_change'])
-        if self.attrs.get('change_default', False):
-            record.cond_default(self.attrs['name'], self.get(record))
         record.on_change_with(self.name)
+        record.autocomplete_with(self.name)
+
+    def domains_get(self, record):
+        screen_domain = domain_inversion(record.group.domain4inversion,
+            self.name, EvalEnvironment(record, False))
+        if isinstance(screen_domain, bool) and not screen_domain:
+            screen_domain = [('id', '=', False)]
+        elif isinstance(screen_domain, bool) and screen_domain:
+            screen_domain = []
+        attr_domain = record.expr_eval(self.attrs.get('domain', []))
+        return screen_domain, attr_domain
 
     def domain_get(self, record):
-        return record.expr_eval(self.attrs.get('domain', []))
+        screen_domain, attr_domain = self.domains_get(record)
+        return localize_domain(screen_domain) + attr_domain
+
+    def validation_domains(self, record):
+        screen_domain, attr_domain = self.domains_get(record)
+        if attr_domain:
+            return screen_domain, screen_domain + unlocalize_domain(attr_domain,
+                self.name)
+        else:
+            return screen_domain, screen_domain
 
     def context_get(self, record, check_load=True, eval_context=True):
-        context = {}
-        context.update(self.parent.context)
+        context = record.context_get().copy()
+        if record.parent:
+            context.update(record.parent.context_get())
         if eval_context:
             context.update(record.expr_eval(self.attrs.get('context', {}),
                 check_load=check_load))
         return context
 
-    def validate(self, record):
+    def validate(self, record, softvalidation=False):
+        if self.attrs.get('readonly'):
+            return True
         res = True
-        if bool(int(self.get_state_attrs(record).get('required') or 0)):
-            if not self.get(record) \
-                    and not bool(int(self.get_state_attrs(record
-                        ).get('readonly') or 0)):
-                res = False
+        self.get_state_attrs(record)['domain_readonly'] = False
+        inverted_domain, domain = self.validation_domains(record)
+        if not softvalidation:
+            if bool(int(self.get_state_attrs(record).get('required') or 0)):
+                if not self.get(record) \
+                        and not bool(int(self.get_state_attrs(record
+                            ).get('readonly') or 0)):
+                    res = False
+        if isinstance(domain, bool):
+            res = res and domain
+        elif domain == [('id', '=', False)]:
+            res = False
+        else:
+            if (isinstance(inverted_domain, list) \
+                and len(inverted_domain) == 1 and inverted_domain[0][1] == '='):
+                # If the inverted domain is so constraint that only one value is
+                # possible we should use it. But we must also pay attention to
+                # the fact that the original domain might be a 'OR' domain and
+                # thus not preventing the modification of fields.
+                leftpart, _, value = inverted_domain[0][:3]
+                setdefault = True
+                if '.' in leftpart:
+                    recordpart, localpart = leftpart.split('.', 1)
+                    original_domain = merge(record.group.domain)
+                    constraintfields = set()
+                    if original_domain[0] == 'AND':
+                        for leaf in localize_domain(original_domain[1:]):
+                            constraintfields.add(leaf[0])
+                    if localpart != 'id' or recordpart not in constraintfields:
+                        setdefault = False
+                if setdefault:
+                    self.set_client(record, value)
+                    self.get_state_attrs(record)['domain_readonly'] = True
+            res = res and eval_domain(domain, EvalEnvironment(record, False))
         self.get_state_attrs(record)['valid'] = res
         return res
 
     def set(self, record, value, modified=False):
         record.value[self.name] = value
         if modified:
-            record.modified = True
             record.modified_fields.setdefault(self.name)
+            record.signal('record-modified')
+            record.signal('record-changed')
         return True
 
     def get(self, record, check_load=True, readonly=True, modified=False):
@@ -81,17 +133,18 @@ class CharField(object):
 
     def set_client(self, record, value, force_change=False):
         internal = record.value.get(self.name, False)
-        prev_modified = record.modified
+        prev_modified_fields = record.modified_fields.copy()
         self.set(record, value)
         if (internal or False) != (record.value.get(self.name, False) or False):
-            record.modified = True
             record.modified_fields.setdefault(self.name)
+            record.signal('record-modified')
             try:
                 self.sig_changed(record)
             except Exception:
                 record.value[self.name] = internal
-                record.modified = prev_modified
+                record.modified_fields = prev_modified_fields
                 return
+            record.validate(softvalidation=True)
             record.signal('record-changed')
 
     def get_client(self, record):
@@ -102,7 +155,8 @@ class CharField(object):
         return res
 
     def set_on_change(self, record, value):
-        return self.set(record, value, modified=True)
+        record.modified_fields.setdefault(self.name)
+        return self.set(record, value, modified=False)
 
     def get_default(self, record):
         return self.get(record)
@@ -117,7 +171,8 @@ class CharField(object):
                 self.get_state_attrs(record)[key] = state_changes[key]
             elif key in self.attrs:
                 self.get_state_attrs(record)[key] = self.attrs[key]
-        if record.group.readonly:
+        if (record.group.readonly
+            or self.get_state_attrs(record).get('domain_readonly')):
             self.get_state_attrs(record)['readonly'] = True
         if 'value' in state_changes:
             value = state_changes['value']
@@ -127,16 +182,13 @@ class CharField(object):
     def get_state_attrs(self, record):
         if self.name not in record.state_attrs:
             record.state_attrs[self.name] = self.attrs.copy()
-        if record.group.readonly:
+        if record.group.readonly or record.readonly:
             record.state_attrs[self.name]['readonly'] = True
         return record.state_attrs[self.name]
 
     def get_timestamp(self, record):
         return {}
 
-    def destroy(self):
-        self.parent = None
-
 
 class SelectionField(CharField):
 
@@ -150,7 +202,7 @@ class SelectionField(CharField):
 class DateTimeField(CharField):
 
     def set_client(self, record, value, force_change=False):
-        if value:
+        if value and not isinstance(value, datetime.datetime):
             value = datetime.datetime(*time.strptime(value, DHM_FORMAT)[:6])
         return super(DateTimeField, self).set_client(record, value,
                 force_change=force_change)
@@ -165,7 +217,7 @@ class DateTimeField(CharField):
 class DateField(CharField):
 
     def set_client(self, record, value, force_change=False):
-        if value:
+        if value and not isinstance(value, datetime.date):
             value = datetime.date(*time.strptime(value, DT_FORMAT)[:3])
         return super(DateField, self).set_client(record, value,
                 force_change=force_change)
@@ -181,19 +233,21 @@ class FloatField(CharField):
 
     def set_client(self, record, value, force_change=False):
         internal = record.value.get(self.name)
-        prev_modified = record.modified
+        prev_modified_fields = record.modified_fields.copy()
         self.set(record, value)
         digits = record.expr_eval(self.attrs.get('digits', (16, 2)))
         if abs(float(internal or 0.0) - float(record.value[self.name] or 0.0)) \
                 >= (10.0**(-int(digits[1]))):
             if not self.get_state_attrs(record).get('readonly', False):
-                record.modified = True
                 record.modified_fields.setdefault(self.name)
+                record.signal('record-modified')
                 try:
                     self.sig_changed(record)
                 except Exception:
                     record.value[self.name] = internal
-                    record.modified = prev_modified
+                    record.modified_fields = prev_modified_fields
+                    return
+                record.validate(softvalidation=True)
                 record.signal('record-changed')
 
 
@@ -202,21 +256,22 @@ class NumericField(CharField):
     def set_client(self, record, value, force_change=False):
         value = Decimal(str(value))
         internal = record.value.get(self.name)
-        prev_modified = record.modified
+        prev_modified_fields = record.modified_fields.copy()
         self.set(record, value)
         digits = record.expr_eval(self.attrs.get('digits', (16, 2)))
         if abs((internal or Decimal('0.0')) - \
                 (record.value[self.name] or Decimal('0.0'))) \
                 >= Decimal(str(10.0**(-int(digits[1])))):
             if not self.get_state_attrs(record).get('readonly', False):
-                record.modified = True
                 record.modified_fields.setdefault(self.name)
+                record.signal('record-modified')
                 try:
                     self.sig_changed(record)
                 except Exception:
                     record.value[self.name] = internal
-                    record.modified = prev_modified
+                    record.prev_modified_fields = prev_modified_fields
                     return
+                record.validate(softvalidation=True)
                 record.signal('record-changed')
 
 
@@ -233,17 +288,18 @@ class BooleanField(CharField):
     def set_client(self, record, value, force_change=False):
         value = bool(value)
         internal = bool(record.value.get(self.name, False))
-        prev_modified = record.modified
+        prev_modified_fields = record.modified_fields.copy()
         self.set(record, value)
         if internal != bool(record.value.get(self.name, False)):
-            record.modified = True
             record.modified_fields.setdefault(self.name)
+            record.signal('record-modified')
             try:
                 self.sig_changed(record)
             except Exception:
                 record.value[self.name] = internal
-                record.modified = prev_modified
+                record.modified_fields = prev_modified_fields
                 return
+            record.validate(softvalidation=True)
             record.signal('record-changed')
 
     def get(self, record, check_load=True, readonly=True, modified=False):
@@ -259,25 +315,44 @@ class M2OField(CharField):
     '''
 
     def get(self, record, check_load=True, readonly=True, modified=False):
-        if record.value.get(self.name):
-            if isinstance(record.value[self.name], (int, basestring, long)):
-                self.set(record, record.value[self.name])
-            if isinstance(record.value[self.name], (int, basestring, long)):
-                return record.value[self.name]
-            return record.value[self.name][0] or False
+        value = record.value.get(self.name)
+        if (record.parent_name == self.name
+                and self.attrs['relation'] == record.group.parent.model_name):
+            value = record.parent.id if record.parent else False
+        if value:
+            if isinstance(value, (int, basestring, long)):
+                self.set(record, value)
+                value = record.value.get(self.name, value)
+            if isinstance(value, (int, basestring, long)):
+                return value
+            return value[0] or False
         return False
 
     def get_client(self, record):
-        if record.value.get(self.name):
-            if isinstance(record.value[self.name], (int, basestring, long)):
-                self.set(record, record.value[self.name])
-            if isinstance(record.value[self.name], (int, basestring, long)):
-                return record.value[self.name]
-            return record.value[self.name][1]
+        value = record.value.get(self.name)
+        if (record.parent_name == self.name
+                and self.attrs['relation'] == record.group.parent.model_name):
+            value = record.parent.id if record.parent else False
+        if value:
+            if isinstance(value, (int, basestring, long)):
+                self.set(record, value)
+                value = record.value.get(self.name, value)
+            if isinstance(value, (int, basestring, long)):
+                return value
+            return value[1]
         return False
 
     def set(self, record, value, modified=False):
-        if value and isinstance(value, (int, long)):
+        if (record.parent_name == self.name
+                and self.attrs['relation'] == record.group.parent.model_name):
+            if record.parent:
+                if 'rec_name' in record.parent.value:
+                    value = (record.parent.id, record.parent.value['rec_name'])
+                else:
+                    value = record.parent.id
+            else:
+                value = False
+        if value and isinstance(value, (int, long)) and value > 0:
             args = ('model', self.attrs['relation'], 'read', value,
                     ['rec_name'], rpc.CONTEXT)
             try:
@@ -288,7 +363,8 @@ class M2OField(CharField):
                 if not result:
                     return
             value = value, result['rec_name']
-        if value and len(value) != 2:
+        if value and (isinstance(value, (int, long))
+                or len(value) != 2):
             value = (False, '')
             record.value[self.name + '.rec_name'] = ''
         else:
@@ -297,23 +373,31 @@ class M2OField(CharField):
             else:
                 record.value[self.name + '.rec_name'] = ''
         record.value[self.name] = value or (False, '')
+        if (record.parent_name == self.name
+                and self.attrs['relation'] == record.group.parent.model_name):
+            if record.parent:
+                if 'rec_name' not in record.parent.value:
+                    record.parent.value['rec_name'] = \
+                            record.value[self.name + '.rec_name']
         if modified:
-            record.modified = True
             record.modified_fields.setdefault(self.name)
+            record.signal('record-modified')
+            record.signal('record-changed')
 
     def set_client(self, record, value, force_change=False):
         internal = record.value.get(self.name) or (False, '')
-        prev_modified = record.modified
+        prev_modified_fields = record.modified_fields.copy()
         self.set(record, value)
         if (internal[0] or False) != (record.value[self.name][0] or False):
-            record.modified = True
             record.modified_fields.setdefault(self.name)
+            record.signal('record-modified')
             try:
                 self.sig_changed(record)
             except Exception:
                 record.value[self.name] = internal
-                record.modified = prev_modified
+                record.modified_fields = prev_modified_fields
                 return
+            record.validate(softvalidation=True)
             record.signal('record-changed')
         elif force_change:
             try:
@@ -321,6 +405,7 @@ class M2OField(CharField):
             except Exception:
                 record.value[self.name] = internal
                 return
+            record.validate(softvalidation=True)
             record.signal('record-changed')
 
     def context_get(self, record, check_load=True, eval_context=True):
@@ -331,32 +416,73 @@ class M2OField(CharField):
                     check_load=check_load)[self.attrs.get('datetime_field')]
         return context
 
+    def validation_domains(self, record):
+        screen_domain, attr_domain = self.domains_get(record)
+        return screen_domain, screen_domain
+
+    def domain_get(self, record):
+        screen_domain, attr_domain = self.domains_get(record)
+        return localize_domain(inverse_leaf(screen_domain), self.name) + attr_domain
+
+    def get_state_attrs(self, record):
+        result = super(M2OField, self).get_state_attrs(record)
+        if (record.parent_name == self.name
+                and self.attrs['relation'] == record.group.parent.model_name):
+            result = result.copy()
+            result['readonly'] = True
+        return result
+
+class O2OField(M2OField):
+    pass
+
 
 class O2MField(CharField):
     '''
     internal = Group of the related objects
     '''
 
-    def __init__(self, parent, attrs):
-        super(O2MField, self).__init__(parent, attrs)
+    def __init__(self, attrs):
+        super(O2MField, self).__init__(attrs)
+        self.in_on_change = False
         self.context = {}
 
+    def sig_changed(self, record):
+        if not self.in_on_change:
+            return super(O2MField, self).sig_changed(record)
+
     def _group_changed(self, group, record):
-        record.parent.modified = True
+        if not record.parent:
+            return
         record.parent.modified_fields.setdefault(self.name)
+        record.parent.signal('record-modified')
         self.sig_changed(record.parent)
+        record.parent.validate(softvalidation=True)
         record.parent.signal('record-changed')
 
+    def _group_list_changed(self, group, signal):
+        if group.model_name == group.parent.model_name:
+            group.parent.group.signal('group-list-changed', signal)
+
+    def _group_cleared(self, group, signal):
+        if group.model_name == group.parent.model_name:
+            group.parent.signal('group-cleared')
+
     def _set_default_value(self, record):
         if record.value.get(self.name) is not None:
             return
         from group import Group
+        parent_name = self.attrs.get('relation_field', '')
         group = Group(self.attrs['relation'], {}, record.window,
                 parent=record,
-                parent_name=self.attrs.get('relation_field', ''),
+                parent_name=parent_name,
+                child_name=self.name,
                 context=self.context,
                 parent_datetime_field=self.attrs.get('datetime_field'))
+        if record.model_name == self.attrs['relation']:
+            group.fields = record.group.fields
         group.signal_connect(group, 'group-changed', self._group_changed)
+        group.signal_connect(group, 'group-list-changed', self._group_list_changed)
+        group.signal_connect(group, 'group-cleared', self._group_cleared)
         record.value[self.name] = group
 
     def get_client(self, record):
@@ -366,23 +492,25 @@ class O2MField(CharField):
     def get(self, record, check_load=True, readonly=True, modified=False):
         if record.value.get(self.name) is None:
             return []
+        record_removed = record.value[self.name].record_removed
+        record_deleted = record.value[self.name].record_deleted
         result = [('add', [])]
         for record2 in record.value[self.name]:
+            if record2 in record_removed or record2 in record_deleted:
+                continue
             if record2.id > 0:
-                values = record2.get(check_load=check_load, get_readonly=readonly,
-                        get_modifiedonly=modified)
-                if record2.is_modified() and values:
+                values = record2.get(check_load=check_load,
+                    get_readonly=readonly, get_modifiedonly=modified)
+                if record2.modified and values:
                     result.append(('write', record2.id, values))
                 result[0][1].append(record2.id)
             else:
                 result.append(('create',
                     record2.get(check_load=check_load, get_readonly=readonly)))
-        if record.value[self.name].record_removed:
-            result.append(('unlink', [x.id for x in \
-                    record.value[self.name].record_removed]))
-        if record.value[self.name].record_deleted:
-            result.append(('delete', [x.id for x in \
-                    record.value[self.name].record_deleted]))
+        if record_removed:
+            result.append(('unlink', [x.id for x in record_removed]))
+        if record_deleted:
+            result.append(('delete', [x.id for x in record_deleted]))
         return result
 
     def get_timestamp(self, record):
@@ -403,24 +531,35 @@ class O2MField(CharField):
         if record.value.get(self.name) is None:
             return []
         for record2 in record.value[self.name]:
-            result.append(record2.get_eval(check_load=check_load))
+            if not (record2.deleted or record2.removed):
+                result.append(record2.get_eval(check_load=check_load))
         return result
 
     def set(self, record, value, modified=False):
         from group import Group
         group = record.value.get(self.name)
+        fields = {}
         if group is not None:
             group.signal_unconnect(group)
+            fields = group.fields
+        elif record.model_name == self.attrs['relation']:
+            fields = record.group.fields
+        parent_name = self.attrs.get('relation_field', '')
         group = Group(self.attrs['relation'], {}, record.window,
-                parent=record, parent_name=self.attrs.get('relation_field', ''),
+                parent=record, parent_name=parent_name,
+                child_name=self.name,
                 context=self.context,
                 parent_datetime_field=self.attrs.get('datetime_field'))
+        group.fields = fields
         record.value[self.name] = group
-        record.value[self.name].load(value, display=False)
+        group.load(value, display=False)
         group.signal_connect(group, 'group-changed', self._group_changed)
+        group.signal_connect(group, 'group-list-changed', self._group_list_changed)
+        group.signal_connect(group, 'group-cleared', self._group_cleared)
         if modified:
-            record.modified = True
             record.modified_fields.setdefault(self.name)
+            record.signal('record-modified')
+            record.signal('record-changed')
 
     def set_client(self, record, value, force_change=False):
         pass
@@ -428,38 +567,51 @@ class O2MField(CharField):
     def set_default(self, record, value, modified=False):
         from group import Group
 
+        # value is a list of id
         if value and len(value) and isinstance(value[0], (int, long)):
             return self.set(record, value, modified=modified)
 
+        group = record.value.get(self.name)
         fields = {}
+        if group is not None:
+            group.signal_unconnect(group)
+            fields = group.fields
+        elif record.model_name == self.attrs['relation']:
+            fields = record.group.fields
+        if fields:
+            fields = dict((fname, field.attrs)
+                for fname, field in fields.iteritems())
+
+        # value is a list of dict
+        fields_dict = {}
         if value and len(value):
             context = self.context_get(record)
             field_names = []
             for val in value:
                 for fieldname in val.keys():
-                    if fieldname not in field_names:
+                    if (fieldname not in field_names
+                            and fieldname not in fields):
                         field_names.append(fieldname)
-            args = ('model', self.attrs['relation'], 'fields_get',
-                    field_names, context)
-            try:
-                fields = rpc.execute(*args)
-            except Exception, exception:
-                fields = common.process_exception(exception, record.window,
-                        *args)
-                if not fields:
-                    return False
+            if field_names:
+                args = ('model', self.attrs['relation'], 'fields_get',
+                        field_names, context)
+                try:
+                    fields_dict = rpc.execute(*args)
+                except Exception, exception:
+                    fields_dict = common.process_exception(exception,
+                            record.window, *args)
+                    if not fields_dict:
+                        return False
 
-        group = record.value.get(self.name)
-        if group is not None:
-            group.signal_unconnect(group)
-        group = Group(
-                self.attrs['relation'], fields, record.window, parent=record,
-                parent_name=self.attrs.get('relation_field', ''),
+        parent_name = self.attrs.get('relation_field', '')
+        group = Group(self.attrs['relation'], fields, record.window,
+                parent=record, parent_name=parent_name, child_name=self.name,
                 context=self.context,
                 parent_datetime_field=self.attrs.get('datetime_field'))
+        group.load_fields(fields_dict)
         if record.value.get(self.name):
             group.record_deleted.extend(x for x in record.value[self.name]
-                    if x.id > 0)
+                if x.id > 0)
             group.record_deleted.extend(record.value[self.name].record_deleted)
             group.record_removed.extend(record.value[self.name].record_removed)
         record.value[self.name] = group
@@ -467,8 +619,9 @@ class O2MField(CharField):
             new_record = record.value[self.name].new(default=False)
             new_record.set_default(vals, modified=modified)
             group.add(new_record)
-        group.signal_connect(group,
-                'group-changed', self._group_changed)
+        group.signal_connect(group, 'group-changed', self._group_changed)
+        group.signal_connect(group, 'group-list-changed', self._group_list_changed)
+        group.signal_connect(group, 'group-cleared', self._group_cleared)
         return True
 
     def set_on_change(self, record, value):
@@ -503,7 +656,8 @@ class O2MField(CharField):
                 if record2 is not None:
                     to_remove.append(record2)
         for record2 in to_remove:
-            record.value[self.name].remove(record2, signal=False)
+            record.value[self.name].remove(record2, signal=False,
+                force_remove=True)
 
         if value and value.get('add') or value.get('update', []):
             record.value[self.name].add_fields(fields, signal=False)
@@ -525,17 +679,23 @@ class O2MField(CharField):
         res = [x.get_default() for x in record.value.get(self.name) or []]
         return res
 
-    def validate(self, record):
+    def validation_domains(self, record):
+        screen_domain, attr_domain = self.domains_get(record)
+        return screen_domain, screen_domain
+
+    def validate(self, record, softvalidation=False):
+        if self.attrs.get('readonly'):
+            return True
         res = True
-        for record2 in record.value[self.name] or []:
+        for record2 in record.value.get(self.name, []):
             if not record2.loaded:
                 continue
             if not record2.validate():
-                if not record2.is_modified():
+                if not record2.modified:
                     record.value[self.name].remove(record2)
                 else:
                     res = False
-        if not super(O2MField, self).validate(record):
+        if not super(O2MField, self).validate(record, softvalidation):
             res = False
         self.get_state_attrs(record)['valid'] = res
         return res
@@ -551,6 +711,10 @@ class O2MField(CharField):
     def get_removed_ids(self, record):
         return [x.id for x in record.value[self.name].record_removed]
 
+    def domain_get(self, record):
+        screen_domain, attr_domain = self.domains_get(record)
+        return localize_domain(inverse_leaf(screen_domain)) + attr_domain
+
 
 class M2MField(O2MField):
 
@@ -563,10 +727,16 @@ class M2MField(O2MField):
     def set(self, record, value, modified=False):
         from group import Group
         group = record.value.get(self.name)
+        fields = {}
         if group is not None:
             group.signal_unconnect(group)
+            fields = group.fields
+        elif record.model_name == self.attrs['relation']:
+            fields = record.group.fields
+        parent_name = self.attrs.get('relation_field', '')
         group = Group(self.attrs['relation'], {}, record.window,
-                parent=record, parent_name=self.attrs.get('relation_field', ''),
+                parent=record, parent_name=parent_name,
+                child_name=self.name,
                 context=self.context,
                 parent_datetime_field=self.attrs.get('datetime_field'))
         if record.value.get(self.name):
@@ -574,11 +744,15 @@ class M2MField(O2MField):
             group.record_deleted.extend(record.value[self.name].record_deleted)
             group.record_removed.extend(record.value[self.name].record_removed)
         record.value[self.name] = group
-        record.value[self.name].load(value, display=False)
+        group.fields = fields
+        group.load(value, display=False)
         group.signal_connect(group, 'group-changed', self._group_changed)
+        group.signal_connect(group, 'group-list-changed', self._group_list_changed)
+        group.signal_connect(group, 'group-cleared', self._group_cleared)
         if modified:
-            record.modified = True
             record.modified_fields.setdefault(self.name)
+            record.signal('record-modified')
+            record.signal('record-changed')
 
 
 class ReferenceField(CharField):
@@ -600,29 +774,33 @@ class ReferenceField(CharField):
 
     def set_client(self, record, value, force_change=False):
         internal = record.value.get(self.name)
-        prev_modified = record.modified
+        prev_modified_fields = record.modified_fields.copy()
         self.set(record, value)
         if (internal or False) != (record.value[self.name] or False):
-            record.modified = True
             record.modified_fields.setdefault(self.name)
+            record.signal('record-modified')
             try:
                 self.sig_changed(record)
             except Exception:
                 record.value[self.name] = internal
-                record.modified = prev_modified
+                record.modified_fields = prev_modified_fields
                 return
+            record.validate(softvalidation=True)
             record.signal('record-changed')
 
     def set(self, record, value, modified=False):
         if not value:
             record.value[self.name] = False
             return
+        if isinstance(value, basestring):
+            model, ref_id = value.split(',')
+            value = model, (ref_id, record.value.get(self.name + '.rec_name'))
         ref_model, (ref_id, ref_str) = value
         if ref_model:
             ref_id = int(ref_id)
             if not ref_id:
                 ref_str = ''
-            if not ref_str and ref_id:
+            if not ref_str and ref_id > 0:
                 args = ('model', ref_model, 'read', ref_id,
                         ['rec_name'], rpc.CONTEXT)
                 try:
@@ -647,8 +825,8 @@ class ReferenceField(CharField):
             if self.name + '.rec_name' in record.value:
                 del record.value[self.name + '.rec_name']
         if modified:
-            record.modified = True
             record.modified_fields.setdefault(self.name)
+            record.signal('record-modified')
 
 TYPES = {
     'char': CharField,
@@ -666,4 +844,5 @@ TYPES = {
     'boolean': BooleanField,
     'datetime': DateTimeField,
     'date': DateField,
+    'one2one': O2OField,
 }
