@@ -6,6 +6,7 @@ from record import Record
 from field import Field, O2MField
 from tryton.signal_event import SignalEvent
 import tryton.common as common
+from tryton.common.domain_inversion import is_leaf
 
 
 class Group(SignalEvent, list):
@@ -17,10 +18,11 @@ class Group(SignalEvent, list):
         if domain is None:
             domain = []
         self.domain = domain
+        self.__domain4inversion = None
         self.lock_signal = False
         self.__window = window
         self.parent = parent
-        self.parent_name = parent_name
+        self.parent_name = parent_name or ''
         self.child_name = child_name
         self.parent_datetime_field = parent_datetime_field
         self._context = context or {}
@@ -36,6 +38,7 @@ class Group(SignalEvent, list):
             self.readonly = True
         self.__id2record = {}
         self.__field_childs = None
+        self.exclude_field = None
 
     def __get_window(self):
         return self.__window
@@ -47,6 +50,29 @@ class Group(SignalEvent, list):
 
     window = property(__get_window, __set_window)
 
+    def clean4inversion(self, domain):
+        "This method will replace non relevant fields for domain inversion"
+        if domain in ([], ()):
+            return []
+        head, tail = domain[0], domain[1:]
+        if head in ('AND', 'OR'):
+            pass
+        elif is_leaf(head):
+            field = head[0]
+            if (field in self.fields
+                    and self.fields[field].attrs.get('readonly')):
+                head = []
+        else:
+            head = self.clean4inversion(head)
+        return [head] + self.clean4inversion(tail)
+
+    def __get_domain4inversion(self):
+        if self.__domain4inversion is None:
+            self.__domain4inversion = self.clean4inversion(self.domain)
+        return self.__domain4inversion
+
+    domain4inversion = property(__get_domain4inversion)
+
     def insert(self, pos, record):
         assert record.group is self
         if pos >= 1:
@@ -228,6 +254,12 @@ class Group(SignalEvent, list):
             self.append(record)
         else:
             self.insert(position, record)
+        for record_rm in self.record_removed:
+            if record_rm.id == record.id:
+                self.record_removed.remove(record)
+        for record_del in self.record_deleted:
+            if record_del.id == record.id:
+                self.record_deleted.remove(record)
         self.current_idx = position
         if modified:
             record.modified_fields.setdefault('id')
