@@ -292,7 +292,10 @@ class Record(SignalEvent):
                 vals = common.process_exception(exception, self.window, *args)
                 if not vals:
                     return
-            if self.parent_name in self.group.fields and self.parent:
+            if (self.parent
+                    and self.parent_name in self.group.fields
+                    and (self.group.fields[self.parent_name].attrs['relation']
+                        == self.group.parent.model_name)):
                 vals[self.parent_name] = self.parent.id
             self.set_default(vals)
         for fieldname, fieldinfo in self.group.fields.iteritems():
@@ -321,6 +324,10 @@ class Record(SignalEvent):
         for field_name, field in self.group.fields.iteritems():
             if fields and field_name not in fields:
                 continue
+            if field.get_state_attrs(self).get('readonly', False):
+                continue
+            if field_name == self.group.exclude_field:
+                continue
             if not field.validate(self, softvalidation):
                 res = False
         return res
@@ -434,7 +441,7 @@ class Record(SignalEvent):
             values[name] = field.get_on_change_value(self, check_load=False)
         if self.parent and self.parent_name:
             values['_parent_' + self.parent_name] = \
-                    common.EvalEnvironment(self.parent, False)
+                    common.EvalEnvironment(self.parent, False, 'on_change')
         for arg in args:
             scope = values
             for i in arg.split('.'):
