@@ -1,10 +1,12 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
+from __future__ import with_statement
 import gtk
 import gettext
 import os
+import tempfile
 from base64 import encodestring, decodestring
-from tryton.common import file_selection, Tooltips
+from tryton.common import file_selection, Tooltips, file_open
 from tryton.config import PIXMAPS_DIR
 from interface import WidgetInterface
 import urllib
@@ -20,6 +22,7 @@ class Image(WidgetInterface):
         super(Image, self).__init__(field_name, model_name, window,
                 attrs=attrs)
 
+        self.filename = attrs.get('filename')
         self.height = int(attrs.get('img_height', 100))
         self.width = int(attrs.get('img_width', 300))
 
@@ -38,37 +41,55 @@ class Image(WidgetInterface):
         self.event.add(self.image)
         self.widget.pack_start(self.event, expand=True, fill=True)
 
-        self.alignment = gtk.Alignment(xalign=0.5, yalign=0.5)
-        self.hbox = gtk.HBox(spacing=0)
-        self.but_add = gtk.Button()
-        img_add = gtk.Image()
-        img_add.set_from_stock('tryton-open', gtk.ICON_SIZE_SMALL_TOOLBAR)
-        self.but_add.set_image(img_add)
-        self.but_add.set_relief(gtk.RELIEF_NONE)
-        self.but_add.connect('clicked', self.sig_add)
-        self.tooltips.set_tip(self.but_add, _('Set Image'))
-        self.hbox.pack_start(self.but_add, expand=False, fill=False)
+        if not attrs.get('readonly'):
+            alignment = gtk.Alignment(xalign=0.5, yalign=0.5)
+            hbox = gtk.HBox(spacing=0)
+            self.but_add = gtk.Button()
+            img_add = gtk.Image()
+            img_add.set_from_stock('tryton-find', gtk.ICON_SIZE_SMALL_TOOLBAR)
+            self.but_add.set_image(img_add)
+            self.but_add.set_relief(gtk.RELIEF_NONE)
+            self.but_add.connect('clicked', self.sig_add)
+            self.tooltips.set_tip(self.but_add, _('Select an Image...'))
+            hbox.pack_start(self.but_add, expand=False, fill=False)
 
-        self.but_save_as = gtk.Button()
-        img_save_as = gtk.Image()
-        img_save_as.set_from_stock('tryton-save', gtk.ICON_SIZE_SMALL_TOOLBAR)
-        self.but_save_as.set_image(img_save_as)
-        self.but_save_as.set_relief(gtk.RELIEF_NONE)
-        self.but_save_as.connect('clicked', self.sig_save_as)
-        self.tooltips.set_tip(self.but_save_as, _('Save As'))
-        self.hbox.pack_start(self.but_save_as, expand=False, fill=False)
+            if self.filename:
+                self.but_open = gtk.Button()
+                img_open = gtk.Image()
+                img_open.set_from_stock('tryton-open', gtk.ICON_SIZE_SMALL_TOOLBAR)
+                self.but_open.set_image(img_open)
+                self.but_open.set_relief(gtk.RELIEF_NONE)
+                self.but_open.connect('clicked', self.sig_open)
+                self.tooltips.set_tip(self.but_open, _('Open...'))
+                hbox.pack_start(self.but_open, expand=False, fill=False)
+            else:
+                self.but_open = None
 
-        self.but_remove = gtk.Button()
-        img_remove = gtk.Image()
-        img_remove.set_from_stock('tryton-clear', gtk.ICON_SIZE_SMALL_TOOLBAR)
-        self.but_remove.set_image(img_remove)
-        self.but_remove.set_relief(gtk.RELIEF_NONE)
-        self.but_remove.connect('clicked', self.sig_remove)
-        self.tooltips.set_tip(self.but_remove, _('Clear'))
-        self.hbox.pack_start(self.but_remove, expand=False, fill=False)
+            self.but_save_as = gtk.Button()
+            img_save_as = gtk.Image()
+            img_save_as.set_from_stock('tryton-save', gtk.ICON_SIZE_SMALL_TOOLBAR)
+            self.but_save_as.set_image(img_save_as)
+            self.but_save_as.set_relief(gtk.RELIEF_NONE)
+            self.but_save_as.connect('clicked', self.sig_save_as)
+            self.tooltips.set_tip(self.but_save_as, _('Save As...'))
+            hbox.pack_start(self.but_save_as, expand=False, fill=False)
 
-        self.alignment.add(self.hbox)
-        self.widget.pack_start(self.alignment, expand=False, fill=False)
+            self.but_remove = gtk.Button()
+            img_remove = gtk.Image()
+            img_remove.set_from_stock('tryton-clear', gtk.ICON_SIZE_SMALL_TOOLBAR)
+            self.but_remove.set_image(img_remove)
+            self.but_remove.set_relief(gtk.RELIEF_NONE)
+            self.but_remove.connect('clicked', self.sig_remove)
+            self.tooltips.set_tip(self.but_remove, _('Clear'))
+            hbox.pack_start(self.but_remove, expand=False, fill=False)
+
+            alignment.add(hbox)
+            self.widget.pack_start(alignment, expand=False, fill=False)
+        else:
+            self.but_add = None
+            self.but_open = None
+            self.but_save_as = None
+            self.but_remove = None
 
         self.tooltips.enable()
 
@@ -76,14 +97,23 @@ class Image(WidgetInterface):
 
         self.update_img()
 
+    @property
+    def filename_field(self):
+        return self.record.group.fields.get(self.filename)
+
     def grab_focus(self):
         return self.image.grab_focus()
 
     def _readonly_set(self, value):
         self._readonly = value
-        self.but_add.set_sensitive(not value)
-        self.but_save_as.set_sensitive(not value)
-        self.but_remove.set_sensitive(not value)
+        if self.but_add:
+            self.but_add.set_sensitive(not value)
+        if self.but_open:
+            self.but_open.set_sensitive(not value)
+        if self.but_save_as:
+            self.but_save_as.set_sensitive(not value)
+        if self.but_remove:
+            self.but_remove.set_sensitive(not value)
 
     def sig_add(self, widget):
         filter_all = gtk.FileFilter()
@@ -102,14 +132,34 @@ class Image(WidgetInterface):
         if filename:
             self.field.set_client(self.record,
                     encodestring(open(filename, 'rb').read()))
+            if self.filename_field:
+                self.filename_field.set_client(self.record,
+                        os.path.basename(filename))
             self.update_img()
 
+    def sig_open(self, widget=None):
+        if not self.filename_field:
+            return
+        dtemp = tempfile.mkdtemp(prefix='tryton_')
+        filename = self.filename_field.get(self.record).replace(
+                os.sep, '_').replace(os.altsep or os.sep, '_')
+        file_path = os.path.join(dtemp, filename)
+        with open(file_path, 'wb') as fp:
+            fp.write(decodestring(self.field.get(self.record)))
+        root, type_ = os.path.splitext(filename)
+        if type_:
+            type_ = type_[1:]
+        file_open(file_path, type_, self.window)
+
     def sig_save_as(self, widget):
-        filename = file_selection(_('Save As...'), parent=self.window,
-                action=gtk.FILE_CHOOSER_ACTION_SAVE)
+        filename = ''
+        if self.filename_field:
+            filename = self.filename_field.get(self.record)
+        filename = file_selection(_('Save As...'), filename=filename,
+                parent=self.window, action=gtk.FILE_CHOOSER_ACTION_SAVE)
         if filename:
-            open(filename, 'wb').write(decodestring(
-                self.field.get_client(self.record)))
+            with open(filename, 'wb') as fp:
+                fp.write(decodestring(self.field.get(self.record)))
 
     def sig_remove(self, widget):
         self.field.set_client(self.record, False)
@@ -154,21 +204,21 @@ class Image(WidgetInterface):
             data = decodestring(value)
 
         pixbuf = None
-        for ftype in ('jpeg', 'gif', 'png', 'bmp'):
+        for ftype in ('jpeg', 'gif', 'png', 'bmp', 'svg'):
             try:
                 loader = gtk.gdk.PixbufLoader(ftype)
                 loader.write(data, len(data))
+                pixbuf = loader.get_pixbuf()
+                loader.close()
             except Exception:
                 continue
-            pixbuf = loader.get_pixbuf()
             if pixbuf:
                 break
         if not pixbuf:
             loader = gtk.gdk.PixbufLoader('png')
             loader.write(NOIMAGE, len(NOIMAGE))
             pixbuf = loader.get_pixbuf()
-
-        loader.close()
+            loader.close()
 
         img_height = pixbuf.get_height()
         if img_height > self.height:
