@@ -122,7 +122,7 @@ class Graph(gtk.DrawingArea):
         self.drawGraph(cx, width, height)
         self.drawAxis(cx, width, height)
         self.drawLegend(cx, width, height)
-        surface.write_to_png(filename)
+        surface.write_to_png(filename.encode('utf-8'))
 
         self.queue_draw()
 
