@@ -84,9 +84,18 @@ class AdaptModelGroup(gtk.GenericTreeModel):
         group = record_pos.group
         pos = group.index(record_pos) + offset
         if group is not record.group:
+            prev_group = record.group
             record.group.remove(record, remove=True, force_remove=True)
+            # Don't remove record from previous group
+            # as the new parent will change the parent
+            # This prevents concurrency conflict
+            record.group.record_removed.remove(record)
             group.add(record)
-            record.modified_fields.setdefault(record.parent_name or 'id')
+            if not record.parent_name:
+                record.modified_fields.setdefault(prev_group.parent_name)
+                record.value[prev_group.parent_name] = False
+            else:
+                record.modified_fields.setdefault(record.parent_name)
         group.move(record, pos)
 
     def move_before(self, record, path):
@@ -105,14 +114,6 @@ class AdaptModelGroup(gtk.GenericTreeModel):
             record.modified_fields.setdefault(record.parent_name or 'id')
         group.move(record, 0)
 
-    def move_end(self, record):
-        group = self.group
-        if group != record.group:
-            record.group.remove(record, remove=True)
-            group.add(record)
-            record.modified_fields.setdefault(record.parent_name or 'id')
-        group.move(record, -1)
-
     def sort(self, ids):
         ids2pos = {}
         pos = 0
@@ -584,7 +585,7 @@ class ViewList(ParserView):
             elif self.children_field:
                 store.move_into(record, path)
         else:
-            store.move_end(record)
+            store.move_after(record, (len(store) - 1,))
         context.drop_finish(False, etime)
         if treeview.sequence:
             record.group.set_sequence(field=treeview.sequence)
@@ -712,7 +713,7 @@ class ViewList(ParserView):
             if last_col and last_col.name in fields:
                 del fields[last_col.name]
 
-            if fields:
+            if fields and any(fields.itervalues()):
                 try:
                     rpc.execute('model', 'ir.ui.view_tree_width', 'set_width',
                             self.screen.model_name, fields, rpc.CONTEXT)
@@ -734,12 +735,16 @@ class ViewList(ParserView):
 
     def __select_changed(self, tree_sel):
         previous_record = self.screen.current_record
+        if previous_record and previous_record not in previous_record.group:
+            previous_record = None
 
         if tree_sel.get_mode() == gtk.SELECTION_SINGLE:
             model, iter_ = tree_sel.get_selected()
             if model and iter_:
                 record = model.get_value(iter_, 0)
                 self.screen.current_record = record
+            else:
+                self.screen.current_record = None
 
         elif tree_sel.get_mode() == gtk.SELECTION_MULTIPLE:
             model, paths = tree_sel.get_selected_rows()
@@ -747,6 +752,8 @@ class ViewList(ParserView):
                 iter_ = model.get_iter(paths[0])
                 record = model.get_value(iter_, 0)
                 self.screen.current_record = record
+            else:
+                self.screen.current_record = None
 
         if hasattr(self.widget_tree, 'editable') \
                 and self.widget_tree.editable \
