@@ -72,7 +72,7 @@ class EditableTreeView(gtk.TreeView):
             idx = (current + i + 1) % len(cols)
             if not cols[idx].get_cell_renderers():
                 continue
-            renderer = cols[idx].get_cell_renderers()[0]
+            renderer = cols[idx].get_cell_renderers()[-1]
             if isinstance(renderer, CellRendererToggle):
                 editable = renderer.get_property('activatable')
             elif isinstance(renderer,
@@ -91,7 +91,7 @@ class EditableTreeView(gtk.TreeView):
             idx = (current - (i + 1)) % len(cols)
             if not cols[idx].get_cell_renderers():
                 continue
-            renderer = cols[idx].get_cell_renderers()[0]
+            renderer = cols[idx].get_cell_renderers()[-1]
             if isinstance(renderer, CellRendererToggle):
                 editable = renderer.get_property('activatable')
             elif isinstance(renderer,
@@ -113,7 +113,7 @@ class EditableTreeView(gtk.TreeView):
     def set_value(self):
         path, column = self.get_cursor()
         model = self.get_model()
-        if not path or not column:
+        if not path or not column or not column.name:
             return True
         record = model.get_value(model.get_iter(path), 0)
         field = record[column.name]
@@ -158,7 +158,8 @@ class EditableTreeView(gtk.TreeView):
             entry.editing_done_id = entry.connect('editing_done',
                     self.on_editing_done)
         if event.keyval in self.leaving_record_events:
-            if not record.validate():
+            fields = self.cells.keys()
+            if not record.validate(fields):
                 invalid_fields = record.invalid_fields
                 col = None
                 for col in self.get_columns():
@@ -191,7 +192,7 @@ class EditableTreeView(gtk.TreeView):
         elif event.keyval in (gtk.keysyms.Return,):
             col = None
             for column in self.get_columns():
-                renderer = column.get_cell_renderers()[0]
+                renderer = column.get_cell_renderers()[-1]
                 if isinstance(renderer, CellRendererToggle):
                     editable = renderer.get_property('activatable')
                 elif isinstance(renderer,
@@ -211,16 +212,10 @@ class EditableTreeView(gtk.TreeView):
                     self.on_editing_done)
         elif event.keyval == gtk.keysyms.Escape:
             if record.id < 0:
-                model.remove(model.get_iter(path))
-                self.screen.current_record = False
-            if not path[0]:
-                self.screen.current_record = False
-            if path[0] == len(self.screen.group) \
-                    and path[0]:
-                path = (path[0] - 1,)
-            self.screen.display()
-            if len(self.screen.group):
-                self.set_cursor(path, column, False)
+                self.screen.remove(delete=False, remove=True,
+                    force_remove=True)
+            else:
+                return False
         elif event.keyval in (gtk.keysyms.F3, gtk.keysyms.F2):
             if isinstance(entry, gtk.Entry):
                 value = entry.get_text()
@@ -242,7 +237,6 @@ class EditableTreeView(gtk.TreeView):
                 entry.set_max_length(int(field.attrs.get('size', 0)))
             # store in the record the entry widget to get the value in set_value
             field.editabletree_entry = entry
-            record.modified = True
             record.modified_fields.setdefault(column.name)
             return False
 
