@@ -1,6 +1,7 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
 
+import gobject
 from editabletree import EditableTreeView
 from tryton.gui.window.view_form.view.interface import ParserInterface
 from tryton.gui.window.win_search import WinSearch
@@ -66,8 +67,10 @@ def sort_model(column, treeview, screen):
 
 class ParserTree(ParserInterface):
 
-    def __init__(self, window, parent=None, attrs=None, screen=None):
-        super(ParserTree, self).__init__(window, parent, attrs, screen)
+    def __init__(self, window, parent=None, attrs=None, screen=None,
+            children_field=None):
+        super(ParserTree, self).__init__(window, parent, attrs, screen,
+            children_field=children_field)
         self.treeview = None
 
     def parse(self, model_name, root_node, fields):
@@ -83,6 +86,7 @@ class ParserTree(ParserInterface):
             treeview.cells = {}
         treeview.sequence = attrs.get('sequence', False)
         treeview.colors = attrs.get('colors', '"black"')
+        treeview.keyword_open = attrs.get('keyword_open', False)
         self.treeview = treeview
         treeview.set_property('rules-hint', True)
         if not self.title:
@@ -124,7 +128,22 @@ class ParserTree(ParserInterface):
                     renderer.connect_after('editing-started', send_keys,
                             treeview)
 
-                col = gtk.TreeViewColumn(fields[fname].attrs['string'], renderer)
+                col = gtk.TreeViewColumn(fields[fname].attrs['string'])
+
+                if 'icon' in node_attrs:
+                    render_pixbuf = gtk.CellRendererPixbuf()
+                    col.pack_start(render_pixbuf, expand=False)
+                    icon = node_attrs['icon']
+                    def setter(column, cell, store, iter):
+                        record = store.get_value(iter, 0)
+                        value = record[icon].get_client(record) or ''
+                        common.ICONFACTORY.register_icon(value)
+                        pixbuf = treeview.render_icon(stock_id=value,
+                                size=gtk.ICON_SIZE_BUTTON, detail=None)
+                        cell.set_property('pixbuf', pixbuf)
+                    col.set_cell_data_func(render_pixbuf, setter)
+
+                col.pack_start(renderer, expand=True)
                 col.name = fname
 
                 hbox = gtk.HBox(False, 2)
@@ -169,13 +188,16 @@ class ParserTree(ParserInterface):
                     col.set_fixed_width(width)
                 #XXX doesn't work well when resize columns
                 #col.set_expand(True)
-                if not treeview.sequence \
-                        and fields[fname].attrs.get('sortable', True):
+                if (not treeview.sequence
+                        and not self.children_field
+                        and fields[fname].attrs.get('sortable', True)):
                     col.connect('clicked', sort_model, treeview, self.screen)
                 col.set_resizable(True)
                 col.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
                 col.set_visible(not node_attrs.get('tree_invisible',
                     fields[fname].attrs.get('tree_invisible', False)))
+                if fname == self.screen.exclude_field:
+                    col.set_visible(False)
                 i = treeview.append_column(col)
                 if 'sum' in node_attrs and fields[fname].attrs['type'] \
                         in ('integer', 'biginteger', 'float', 'numeric',
@@ -250,6 +272,10 @@ class Char(object):
         if isinstance(cell, CellRendererToggle):
             cell.set_active(bool(text))
         else:
+            cell.set_sensitive(not (record.deleted or record.removed))
+            if isinstance(cell,
+                (CellRendererText, CellRendererDate, CellRendererCombo)):
+                cell.set_property('strikethrough', record.deleted)
             cell.set_property('text', text)
             fg_color = self.get_color(record)
             cell.set_property('foreground', fg_color)
@@ -267,7 +293,7 @@ class Char(object):
         else:
             align = 0
 
-        states = ('invisible')
+        states = ('invisible',)
         if hasattr(self.treeview, 'editable') \
                 and self.treeview.editable:
             states = ('readonly', 'required', 'invisible')
@@ -293,7 +319,8 @@ class Char(object):
                     cell.set_property('background-set', False)
                 else:
                     cell.set_property('background-set', True)
-                    cell.set_property('foreground-set', True)
+                    cell.set_property('foreground-set',
+                        not (record.deleted or record.removed))
 
             if isinstance(cell, CellRendererToggle):
                 cell.set_property('activatable', not readonly)
@@ -312,6 +339,8 @@ class Char(object):
         raise NotImplementedError
 
     def get_textual_value(self, record):
+        if not record:
+            return ''
         return record[self.field_name].get_client(record) or ''
 
     def value_from_text(self, record, text):
@@ -535,20 +564,17 @@ class M2O(Char):
                 return True, searched
             return False, False
         screen = Screen(relation, self.window, domain=domain, context=context,
-                view_type=['form'])
+                mode=['form'])
         if obj_id:
             screen.load([obj_id])
             win = WinForm(screen, self.window)
         else:
             win = WinForm(screen, self.window, new=True)
         value = False
-        while win.run():
+        if win.run():
             if screen.save_current():
                 value = (screen.current_record.id,
                         screen.current_record.rec_name())
-                break
-            else:
-                screen.display()
         win.destroy()
         if value:
             return True, value
@@ -572,6 +598,9 @@ class M2O(Char):
         else:
             return False, None
 
+class O2O(M2O):
+    pass
+
 class UnsettableColumn(Exception):
 
     def __init__(self):
@@ -597,7 +626,7 @@ class O2M(Char):
         relation = field.attrs['relation']
         context = field.context_get(record)
 
-        screen = Screen(relation, self.window, view_type=['tree', 'form'],
+        screen = Screen(relation, self.window, mode=['tree', 'form'],
                 exclude_field=field.attrs.get('relation_field'))
         screen.group = group
         win = WinForm(screen, self.window, view_type='tree', context=context)
@@ -675,65 +704,88 @@ class Selection(Char):
     def __init__(self, *args):
         super(Selection, self).__init__(*args)
         self.renderer = CellRendererCombo()
-        selection_data = gtk.ListStore(str, str)
+        self.renderer.connect('editing-started', self.editing_started)
+        self._last_domain = None
+        self._domain_cache = {}
         selection = self.attrs.get('selection', [])[:]
-        self.selection = selection[:]
-        if 'relation' in self.attrs:
-            args = ('model', self.attrs['relation'], 'search_read',
-                    self.attrs.get('domain', []), 0, None, None, ['rec_name'],
-                    rpc.CONTEXT)
+        if not isinstance(selection, (list, tuple)):
             try:
-                result = rpc.execute(*args)
+                selection = rpc.execute('model',
+                        self.model_name, selection, rpc.CONTEXT)
             except Exception, exception:
-                result = common.process_exception(exception, self.window, *args)
-                if not result:
-                    result = []
-            selection = [(x['id'], x['rec_name']) for x in result]
-            self.selection = selection[:]
-        else:
-            if not isinstance(selection, (list, tuple)):
-                try:
-                    selection = rpc.execute('model',
-                            self.model_name, selection, rpc.CONTEXT)
-                except Exception, exception:
-                    common.process_exception(exception, self.window)
-                    selection = []
-                self.selection = selection[:]
-
-            for dom in common.filter_domain(self.attrs.get('domain', [])):
-                if dom[1] in ('=', '!='):
-                    todel = []
-                    for i in xrange(len(selection)):
-                        if (dom[1] == '=' \
-                                and selection[i][0] != dom[2]) \
-                                or (dom[1] == '!=' \
-                                and selection[i][0] == dom[2]):
-                            todel.append(i)
-                    for i in todel[::-1]:
-                        del selection[i]
-
+                common.process_exception(exception, self.window)
+                selection = []
+        self.selection = selection[:]
         if self.attrs.get('sort', True):
             selection.sort(lambda x, y: cmp(x[1], y[1]))
-        self._selection = selection
-        for i in selection:
-            selection_data.append(i)
-        self.renderer.set_property('model', selection_data)
-        self.renderer.set_property('text-column', 1)
+        self.renderer.set_property('model', self.get_model(selection))
+        self.renderer.set_property('text-column', 0)
+
+    def get_model(self, selection):
+        model = gtk.ListStore(gobject.TYPE_STRING)
+        self._selection = {}
+        lst = []
+        for (value, name) in selection:
+            name = str(name)
+            lst.append(name)
+            self._selection[name] = value
+            i = model.append()
+            model.set(i, 0, name)
+        return model
 
     def get_textual_value(self, record):
+        self.update_selection(record)
         value = record[self.field_name].get(record)
         if isinstance(value, (list, tuple)):
             value = value[0]
         return dict(self.selection).get(value, '')
 
     def value_from_text(self, record, text):
-        res = False
-        for val, txt in self._selection:
-            if txt[:len(text)].lower() == text.lower():
-                if len(txt) == len(text):
-                    return val
-                res = val
-        return res
+        return self._selection.get(text, False), text
+
+    def editing_started(self, cell, editable, path):
+        store = self.treeview.get_model()
+        record = store.get_value(store.get_iter(path), 0)
+        self.update_selection(record)
+        model = self.get_model(self.selection)
+        editable.set_model(model)
+        editable.set_text_column(0)
+        completion = gtk.EntryCompletion()
+        #Only available in PyGTK 2.6 and above.
+        if hasattr(completion, 'set_inline_selection'):
+            completion.set_inline_selection(True)
+        completion.set_model(model)
+        editable.get_child().set_completion(completion)
+        completion.set_text_column(0)
+        return False
+
+    def update_selection(self, record):
+        if 'relation' not in self.attrs:
+            return
+        field = record[self.field_name]
+        domain = field.domain_get(record)
+        if str(domain) in self._domain_cache:
+            self.selection = self._domain_cache[str(domain)]
+            self._last_domain = domain
+        if domain != self._last_domain:
+            args = ('model', self.attrs['relation'], 'search_read', domain,
+                0, None, None, ['rec_name'], rpc.CONTEXT)
+            try:
+                result = rpc.execute(*args)
+            except Exception, exception:
+                result = common.process_exception(exception, self.window, *args)
+
+            if isinstance(result, list):
+                selection = [(x['id'], x['rec_name']) for x in result]
+                selection.append((False, ''))
+                self._last_domain = domain
+                self._domain_cache[str(domain)] = selection
+            else:
+                selection = []
+                self._last_domain = None
+        else:
+            selection = self.selection
+        self.selection = selection[:]
 
 
 class Reference(Char):
@@ -886,7 +938,7 @@ class Button(object):
                             }, self.window, context=ctx)
                 else:
                     raise Exception('Unallowed button type')
-                self.screen.reload(writen=True)
+                self.screen.reload(written=True)
             else:
                 self.screen.display()
 
@@ -911,4 +963,5 @@ CELLTYPES = {
     'sip': Char,
     'progressbar': ProgressBar,
     'reference': Reference,
+    'one2one': O2O,
 }
